@@ -82,35 +82,35 @@ static int ipkg_configure_cmd(ipkg_conf_t *conf, int argc, char **argv);
 /* XXX: CLEANUP: The usage strings should be incorporated into this
    array for easier maintenance */
 static ipkg_cmd_t cmds[] = {
-     {"update", 0, ipkg_update_cmd}, 
-     {"upgrade", 0, ipkg_upgrade_cmd},
-     {"list", 0, ipkg_list_cmd},
-     {"list_installed", 0, ipkg_list_installed_cmd},
-     {"info", 0, ipkg_info_cmd},
-     {"flag", 1, ipkg_flag_cmd},
-     {"status", 0, ipkg_status_cmd},
-     {"install_pending", 0, ipkg_install_pending_cmd},
-     {"install", 1, ipkg_install_cmd},
-     {"remove", 1, ipkg_remove_cmd},
-     {"purge", 1, ipkg_purge_cmd},
-     {"configure", 0, ipkg_configure_cmd},
-     {"files", 1, ipkg_files_cmd},
-     {"search", 1, ipkg_search_cmd},
-     {"download", 1, ipkg_download_cmd},
-     {"compare_versions", 1, ipkg_compare_versions_cmd},
-     {"compare-versions", 1, ipkg_compare_versions_cmd},
-     {"print-architecture", 0, ipkg_print_architecture_cmd},
-     {"print_architecture", 0, ipkg_print_architecture_cmd},
-     {"print-installation-architecture", 0, ipkg_print_architecture_cmd},
-     {"print_installation_architecture", 0, ipkg_print_architecture_cmd},
-     {"depends", 1, ipkg_depends_cmd},
-     {"whatdepends", 1, ipkg_whatdepends_cmd},
-     {"whatdependsrec", 1, ipkg_whatdepends_recursively_cmd},
-     {"whatrecommends", 1, ipkg_whatrecommends_cmd},
-     {"whatsuggests", 1, ipkg_whatsuggests_cmd},
-     {"whatprovides", 1, ipkg_whatprovides_cmd},
-     {"whatreplaces", 1, ipkg_whatreplaces_cmd},
-     {"whatconflicts", 1, ipkg_whatconflicts_cmd},
+     {"update", 0, (ipkg_cmd_fun_t)ipkg_update_cmd}, 
+     {"upgrade", 0, (ipkg_cmd_fun_t)ipkg_upgrade_cmd},
+     {"list", 0, (ipkg_cmd_fun_t)ipkg_list_cmd},
+     {"list_installed", 0, (ipkg_cmd_fun_t)ipkg_list_installed_cmd},
+     {"info", 0, (ipkg_cmd_fun_t)ipkg_info_cmd},
+     {"flag", 1, (ipkg_cmd_fun_t)ipkg_flag_cmd},
+     {"status", 0, (ipkg_cmd_fun_t)ipkg_status_cmd},
+     {"install_pending", 0, (ipkg_cmd_fun_t)ipkg_install_pending_cmd},
+     {"install", 1, (ipkg_cmd_fun_t)ipkg_install_cmd},
+     {"remove", 1, (ipkg_cmd_fun_t)ipkg_remove_cmd},
+     {"purge", 1, (ipkg_cmd_fun_t)ipkg_purge_cmd},
+     {"configure", 0, (ipkg_cmd_fun_t)ipkg_configure_cmd},
+     {"files", 1, (ipkg_cmd_fun_t)ipkg_files_cmd},
+     {"search", 1, (ipkg_cmd_fun_t)ipkg_search_cmd},
+     {"download", 1, (ipkg_cmd_fun_t)ipkg_download_cmd},
+     {"compare_versions", 1, (ipkg_cmd_fun_t)ipkg_compare_versions_cmd},
+     {"compare-versions", 1, (ipkg_cmd_fun_t)ipkg_compare_versions_cmd},
+     {"print-architecture", 0, (ipkg_cmd_fun_t)ipkg_print_architecture_cmd},
+     {"print_architecture", 0, (ipkg_cmd_fun_t)ipkg_print_architecture_cmd},
+     {"print-installation-architecture", 0, (ipkg_cmd_fun_t)ipkg_print_architecture_cmd},
+     {"print_installation_architecture", 0, (ipkg_cmd_fun_t)ipkg_print_architecture_cmd},
+     {"depends", 1, (ipkg_cmd_fun_t)ipkg_depends_cmd},
+     {"whatdepends", 1, (ipkg_cmd_fun_t)ipkg_whatdepends_cmd},
+     {"whatdependsrec", 1, (ipkg_cmd_fun_t)ipkg_whatdepends_recursively_cmd},
+     {"whatrecommends", 1, (ipkg_cmd_fun_t)ipkg_whatrecommends_cmd},
+     {"whatsuggests", 1, (ipkg_cmd_fun_t)ipkg_whatsuggests_cmd},
+     {"whatprovides", 1, (ipkg_cmd_fun_t)ipkg_whatprovides_cmd},
+     {"whatreplaces", 1, (ipkg_cmd_fun_t)ipkg_whatreplaces_cmd},
+     {"whatconflicts", 1, (ipkg_cmd_fun_t)ipkg_whatconflicts_cmd},
 };
 
 int ipkg_state_changed;
@@ -188,6 +188,7 @@ static int ipkg_update_cmd(ipkg_conf_t *conf, int argc, char **argv)
 	       ipkg_message(conf, IPKG_ERROR,
 			    "%s: ERROR: %s exists, but is not a directory\n",
 			    __FUNCTION__, lists_dir);
+	       free(lists_dir);
 	       return EINVAL;
 	  }
 	  err = file_mkdir_hier(lists_dir, 0755);
@@ -195,6 +196,7 @@ static int ipkg_update_cmd(ipkg_conf_t *conf, int argc, char **argv)
 	       ipkg_message(conf, IPKG_ERROR,
 			    "%s: ERROR: failed to make directory %s: %s\n",
 			    __FUNCTION__, lists_dir, strerror(errno));
+	       free(lists_dir);
 	       return EINVAL;
 	  }	
      } 
@@ -255,6 +257,7 @@ static int ipkg_update_cmd(ipkg_conf_t *conf, int argc, char **argv)
 	  free(url);
 	  free(list_file_name);
      }
+     free(lists_dir);
 
 #ifdef CONFIG_CLEAR_SW_INSTALL_FLAG
 #warning here
@@ -346,6 +349,7 @@ int ipkg_configure_packages(ipkg_conf_t *conf, char *pkg_name)
 	  }
      }
 
+     pkg_vec_free(all);
      return 0;
 }
 
@@ -364,7 +368,7 @@ static int ipkg_install_cmd(ipkg_conf_t *conf, int argc, char **argv)
 {
      int i;
      char *arg;
-     int err;
+     int err=0;
 
      global_conf = conf;
      signal(SIGINT, sigint_handler);
@@ -548,6 +552,7 @@ static int ipkg_list_cmd(ipkg_conf_t *conf, int argc, char **argv)
 	  }
 #endif
      }
+     pkg_vec_free(available);
 
      return 0;
 }
@@ -1062,6 +1067,8 @@ static int ipkg_what_depends_conflicts_cmd(ipkg_conf_t *conf, enum what_field_ty
 	  case WHATCONFLICTS: rel_str = "conflicts with"; break;
 	  case WHATSUGGESTS: rel_str = "suggests"; break;
 	  case WHATRECOMMENDS: rel_str = "recommends"; break;
+	  case WHATPROVIDES: rel_str = "provides"; break;
+	  case WHATREPLACES: rel_str = "replaces"; break;
 	  }
      
 	  if (conf->query_all)
